(require 'cl)
(require 'easymenu)
(require 'cleite-vars)
(require 'cleite-items)

(defvar cleite:article:buffer-name "*cleite article*")

(defvar cleite:article:mode-map
  (let ((map (make-sparse-keymap)))
	(set-keymap-parent map cleite:global-map)
	(define-key map "\r" 'cleite:article:browse)
	(define-key map "U"  'cleite:article:copy-url)
	(define-key map "j"  'cleite:article:scroll-up-a-bit)
	(define-key map "k"  'cleite:article:scroll-down-a-bit)
	(define-key map "N"  'cleite:article:go-to-next-unread)
	(define-key map "P"  'cleite:article:go-to-prev-unread)
	(define-key map "n"  'cleite:article:go-to-next)
	(define-key map "p"  'cleite:article:go-to-prev)
    (define-key map "q"  'cleite:article:quit)
	map)
  "Keymap for cleite:article:mode")

(defvar cleite:article:mode-menu-spec
  '("Article"
	["Browse article"                cleite:article:browse           ]
	["Copy URL..."                   cleite:article:copy-url         ]
	"----"
	["Scroll up a bit"               cleite:article:scroll-up-a-bit  ]
	["Scroll down a bit"             cleite:article:scroll-down-a-bit]
	["Go to next unread article"     cleite:article:go-to-next-unread]
	["Go to previous unread article" cleite:article:go-to-prev-unread]
	["Go to next article"            cleite:article:go-to-next       ]
	["Go to previous article"        cleite:article:go-to-prev       ]
	"----"
	["Close article"                 cleite:article:quit             ]))

(defvar cleite:article:mode-hook nil
  "Hook for cleite:article:mode")

(defvar cleite:article nil)
(defvar cleite:article:feed-uri nil)
(defvar cleite:article:item-uri nil)

(defvar cleite:article:window nil)

(easy-menu-define cleite:article:mode-menu
  cleite:article:mode-map
  "Menu used in cleite:article:mode"
  cleite:article:mode-menu-spec)


(defun* cleite:article:current-item-uri (&key clear)
  (when clear
	(setq cleite:article:item-uri nil))
  cleite:article:item-uri)


(defun cleite:article:show-status (status)
  (if (or (null status)
		  (= 0 (length status)))
	  (setq mode-name "Cleite Article")
	(setq mode-name (format "Cleite Article:%s" status)))

  (force-mode-line-update)
  (sit-for 0))


;;;###autoload
(defun cleite:article:mode ()
  "Major mode for Cleite article view
\\{cleite:article:mode-map}"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'cleite:article:mode)
  (cleite:article:show-status nil)
  (use-local-map cleite:article:mode-map)
  (easy-menu-add cleite:article:mode-menu)
  (run-hooks 'cleite:article:mode-hook))


;;;###autoload
(defun cleite:article (feed-uri item-uri)

  ;; feeds バッファと items バッファのみ表示
  (cleite:items feed-uri)

  ;; items バッファを表示してゐるウインドウを上下に分割する。
  (split-window-vertically cleite:items:window-height)
  (other-window 1)
  (setq cleite:article:window (selected-window))

  ;; article バッファがあれば呼び出し、無ければ作る。
  (if (get-buffer cleite:article:buffer-name)
	  (progn
		(switch-to-buffer cleite:article:buffer-name)
		;; もし指定された url が現在選ばれてゐるものと違ふなら、ここで
		;; refresh する。
		(unless (and (equal feed-uri cleite:article:feed-uri)
					 (equal item-uri cleite:article:item-uri))

		  (setq cleite:article:feed-uri feed-uri)
		  (setq cleite:article:item-uri item-uri)
		  (cleite:article:refresh)))

	(switch-to-buffer (get-buffer-create cleite:article:buffer-name))
	(cleite:article:mode)

	(setq cleite:article:feed-uri feed-uri)
	(setq cleite:article:item-uri item-uri)
	(cleite:article:refresh)))


(defun cleite:article:refresh ()
  (interactive)
  (cleite:article:show-status "Loading...")

  (setq cleite:article (cleite:call 'feed-item:get
									`((feed-uri . ,cleite:article:feed-uri)
									  (item-uri . ,cleite:article:item-uri))))
  (cleite:article:rebuild-buffer)

  (cleite:article:show-status nil))


(defun cleite:article:rebuild-buffer ()
  (setq buffer-read-only nil)
  (erase-buffer)

  (let* ((title       (cdr (assq 'title cleite:article)))
		 (date        (if (assq 'date cleite:article)
						  (cdr (assq 'date cleite:article))
						nil))
		 (description (cdr (assq 'description cleite:article))))

	;; モード行
	(setq mode-line-buffer-identification title)

	;; 題名
	(insert (cleite:faced-string "Title: " 'cleite:face:header-key))
	(insert (cleite:faced-string title     'cleite:face:header-value))
	(insert "\n")

	;; 時刻
	(when date
	  (insert (cleite:faced-string "Date : " 'cleite:face:header-key))
	  (insert (cleite:faced-string (let* ((high (/ date (expt 2 16)))
										  (low  (- date (* high (expt 2 16)))))
									 (current-time-string (list (truncate high)
																(truncate low))))
								   'cleite:face:header-value))
	  (insert "\n"))

	(insert "\n")

	;; 本文
	(let ((before (point)))
	  (insert description)
	  (fill-region before (point))))

  (setq buffer-read-only t)
  (goto-char (point-min)))


(defun cleite:article:quit (&optional kill)
  (interactive "P")
  
  (block nil
	(let ((buf (get-buffer cleite:article:buffer-name)))

	  (if (null buf)
		  (return nil))
	
	  (delete-windows-on buf)

	  (if kill
		  ;; ただバッファを殺すだけ
		  (kill-buffer buf)

		;; items バッファを更新
		(let ((feed-uri cleite:article:feed-uri)
			  (item-uri cleite:article:item-uri))
		  (setq cleite:article:item-uri nil)
		  (cleite:items feed-uri :refresh item-uri)))))

  (setq cleite:article nil)
  (setq cleite:article:feed-uri nil)
  (setq cleite:article:item-uri nil))


(defun cleite:article:browse ()
  (interactive)
  (browse-url cleite:article:item-uri))


(defun cleite:article:copy-url ()
  (interactive)
  (let* ((url    cleite:article:item-uri)
		 (title  (cdr (assq 'title cleite:article)))
		 (choice (cleite:read-char-with-retry
				  (format "Copy [u]rl / Copy [t]itle / Copy [b]oth: `%s': " title)
				  '(?u ?t ?b))))

	(let ((str (cond ((eq choice ?u)
					  url)
					  
					 ((eq choice ?t)
					  title)

					 ((eq choice ?b)
					  (format "%s\n%s" title url)))))
	  (kill-new str)
	  (message "Copied: %s" str))))


(defun cleite:article:scroll-up-a-bit ()
  (interactive)
  (scroll-up 1))


(defun cleite:article:scroll-down-a-bit ()
  (interactive)
  (scroll-down 1))


(defun cleite:article:go-to-next ()
  (interactive)
  (when (cleite:items:go-to-next)
	
	(cleite:items:on-items-window
	  (cleite:items:select))
	
	(select-window (get-buffer-window cleite:article:buffer-name))))


(defun cleite:article:go-to-prev ()
  (interactive)
  (when (cleite:items:go-to-prev)
	
	(cleite:items:on-items-window
	  (cleite:items:select))

	(select-window (get-buffer-window cleite:article:buffer-name))))


(defun cleite:article:go-to-next-unread ()
  (interactive)
  (when (cleite:items:go-to-next-unread)
	
	(cleite:items:on-items-window
	  (cleite:items:select))

	(select-window (get-buffer-window cleite:article:buffer-name))))


(defun cleite:article:go-to-prev-unread ()
  (interactive)
  (when (cleite:items:go-to-prev-unread)
	
	(cleite:items:on-items-window
	  (cleite:items:select))

	(select-window (get-buffer-window cleite:article:buffer-name))))


(provide 'cleite-article)